# -*- coding: utf-8 -*-

import sys
import os


"""
Set django environment
"""
sys.path.append(os.path.join(os.path.dirname(__file__), '../schoolsproject'))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "schoolsproject.settings.dev")

from schools import models
import django
from django.db.models import Avg


def get_aggregated_info_for_city(city):
    """
    Get aggregated info for city
    :param city: City to get aggregation info for
    :return: dict items with aggregation info for city
    """
    return models.School.objects.filter(city=city).aggregate(
        average_receipts=Avg('receipts'),
        average_high_education_index=Avg('high_education_index'),
        average_salary=Avg('average_salary'),
        average_computer_index=Avg('computer_index'),
        average_ege_index=Avg('ege_index'),
        average_payments_per_student=Avg('payments_per_student')
    )


def fill_city_info(city):
    """
    Calculate and fill city info
    :param city: City to fill info
    """
    for (key, value) in get_aggregated_info_for_city(city).items():
        setattr(city, key,
                round(value) if key in ['average_receipts', 'average_salary', 'average_payments_per_student'] else round(value, 1))
    city.save()


def fill_school_payments_per_student(school):
    """
    Fill school payments per student for schools
    :param school:
    :return:
    """
    if school.payments is None or school.payments <= 0:
        return
    if school.number_of_students_index is None or school.number_of_students_index <= 0:
        return
    school.payments_per_student = school.payments / school.number_of_students_index
    school.save()


def get_average_parameter(city, parameter):
    """
    Get aggregated averaged parameter
    :param city:
    :param parameter: string
    :return:
    """
    return models.School.objects.filter(city=city).aggregate(
        average_parameter=Avg(parameter)
    ).get('average_parameter')


def check_school_receipts(school, average_receipts, coefficient):
    """
    Check school's receipts
    :param school:
    :param average_receipts:
    :param coefficient:
    :return:
    """
    if school.receipts is None or school.receipts <= 0:
        return
    if school.receipts > average_receipts * coefficient:
        print("Receipts in big. Set to NULL")
        school.receipts = None
        school.save()


def check_school_payments(school, average_payments, coefficient):
    """
    Check school's payments
    :param school:
    :param average_payments:
    :param coefficient:
    :return:
    """
    if school.payments is None or school.payments <= 0:
        return
    if school.payments > average_payments * coefficient:
        print("Payments is big. Set to NULL")
        school.payments = None
        school.save()


def check_school_average_salary(school, average_salary, coefficient):
    """
    Check school's average salary
    :param school:
    :param average_salary:
    :param coefficient:
    :return:
    """
    if school.average_salary is None or school.average_salary <= 0:
        return
    if school.average_salary > average_salary * coefficient:
        print("Average salary is big. Set to NULL")
        school.average_salary = None
        school.save()


def fill_schools_info_for_city(city):
    """
    Fill payments per student for all schools in the city
    :param city
    """
    schools = models.School.objects.filter(city=city)
    average_payments = get_average_parameter(city, 'payments')
    average_receipts = get_average_parameter(city, 'receipts')
    average_salary = get_average_parameter(city, 'average_salary')
    for school in schools:
        print("Process school " + school.name)
        check_school_payments(school, average_payments, 3)
        check_school_receipts(school, average_receipts, 3)
        check_school_average_salary(school, average_salary, 3)
        fill_school_payments_per_student(school)


def process():
    """
    Main function
    """
    cities = models.City.objects.all()
    for city in cities:
        print("Process city " + city.name)
        fill_schools_info_for_city(city)
        fill_city_info(city)


if __name__ == "__main__":
    django.setup()
    process()