from itertools import chain
from django.shortcuts import render, get_object_or_404
from django.core import serializers
from django.http import HttpResponse
from django.views.decorators.gzip import gzip_page
from .models import City, School


def home(request):
    """
    Home page
    :param request:
    :return: HttpResponse
    """
    city = get_object_or_404(City, alias="barnaul")
    return render(request, 'home.html', {'city': city})


def compare(request):
    """
    Compare page
    :param request:
    :return: HttpResponse
    """
    return render(request, 'compare.html')


def get_top_and_bottom_schools(schools):
    """
    Get top and bottom schools ordered and grouped by parameter from max to min [3 top schools, 3 bottom schools]
    :param schools:
    :return: list of schools
    """
    top_schools = schools[:5]
    bottom_schools = reversed(schools.reverse()[:5])
    return list(chain(top_schools, bottom_schools))


def get_serialized_schools(schools, field):
    """
    Get json serialized schools
    :param schools: list of schools
    :params field: school model field
    :return: json serialized schools
    """
    return serializers.serialize("json", schools, fields=('name', field,))


def get_json_http_response(data):
    """
    Get http response with schools with content-type = json
    :param data: json data
    :return: HttpResponse
    """
    return HttpResponse(data, content_type="application/json")


def get_json_schools(schools, field):
    """
    Get json http response with serialized top and bottom schools by parameter
    Shortcut for all get schools functions
    :param schools: school list
    :param field: school model field
    :return: HttpResponse
    """
    return get_json_http_response(
        get_serialized_schools(
            get_top_and_bottom_schools(schools),
            field
        )
    )


def receipts(request):
    """
    Get schools grouped by receipt
    :param request:
    :return: HttpResponse
    """
    return get_json_schools(
        School.objects.exclude(receipts__isnull=True).order_by('-receipts'),
        'receipts'
    )


def salaries(request):
    """
    Get schools grouped by average salary
    :param request:
    :return: HttpResponse
    """
    return get_json_schools(
        School.objects.exclude(average_salary__isnull=True).order_by('-average_salary'),
        'average_salary'
    )


def payments_per_student(request):
    """
    Get schools grouped by payments_per_student
    :param request:
    :return: HttpResponse
    """
    return get_json_schools(
        School.objects.exclude(payments_per_student__isnull=True).order_by('-payments_per_student'),
        'payments_per_student'
    )


def high_education_index(request):
    """
    Get schools grouped by high education index
    :param request:
    :return: HttpResponse
    """
    return get_json_schools(
        School.objects.exclude(high_education_index__isnull=True).order_by('-high_education_index'),
        'high_education_index'
    )


def ege_index(request):
    """
    Get schools grouped by ege index
    :param request:
    :return: HttpResponse
    """
    return get_json_schools(
        School.objects.exclude(ege_index__isnull=True).order_by('-ege_index'),
        'ege_index'
    )


def computer_index(request):
    """
    Get schools grouped by computer index
    :param request:
    :return: HttpResponse
    """
    return get_json_schools(
        School.objects.exclude(computer_index__isnull=True).order_by('computer_index'),
        'computer_index'
    )


def full_receipts(request):
    return get_json_http_response(
        get_serialized_schools(
            School.objects.all().order_by('-receipts'),
            'receipts'
        )
    )


def full_salaries(request):
    return get_json_http_response(
        get_serialized_schools(
            School.objects.all().order_by('-average_salary'),
            'average_salary'
        )
    )


def full_computers(request):
    return get_json_http_response(
        get_serialized_schools(
            list(
                chain(
                    School.objects.exclude(computer_index__isnull=True).order_by('computer_index'),
                    School.objects.exclude(computer_index__isnull=False),
                )
            ),
            'computer_index'
        )
    )


def full_ege(request):
    return get_json_http_response(
        get_serialized_schools(
            School.objects.all().order_by('-ege_index'),
            'ege_index'
        )
    )


def full_high_education_teachers(request):
    return get_json_http_response(
        get_serialized_schools(
            School.objects.all().order_by('-high_education_index'),
            'high_education_index'
        )
    )


def full_payments_per_student(request):
    return get_json_http_response(
        get_serialized_schools(
            School.objects.all().order_by('-payments_per_student'),
            'payments_per_student'
        )
    )

@gzip_page
def coordinates(request):
    """
    Get coordinates
    :param request:
    :return: HttpResponse
    """
    return get_json_http_response(
        serializers.serialize("json", School.objects.all(), fields=('name', 'latitude', 'longitude',))
    )


def get_school(request, pk):
    """
    Get school info by primary key
    :param request:
    :param pk: school primary key
    :return: HttpResponse
    """
    school = get_object_or_404(School, pk=pk)
    return get_json_http_response(
        serializers.serialize("json", [school])
    )