(function($, ko, store) {


	function initCharts() {
		initReceiptChart();
		initSalariesChart();
		initPaymentsPerStudentChart();
		initHighEducationIndexChart();
		initEgeIndexChart();
		initComputerIndexChart();
	}

	/**
	 * Get canvas context by canvas element name
	 * @param canvasElementName {string}
	 * @returns {CanvasRenderingContext2D}
	 */
	function getContext(canvasElementName) {
		var canvas = document.querySelector(canvasElementName);
		if (!canvas) {
			return null;
		}
		return canvas.getContext('2d');
	}

	function initChart(parameter, canvasElement, field) {
		var ctx = getContext(canvasElement);
		if (!ctx) {
			return;
		}
		store.getSchoolsByParameter(parameter, function(data) {
			field = field || parameter;
			var field_data = data.map(function(school) {
				return Math.round(school['fields'][field]);
			});
			var chartData = {
				labels: data.map(function(school) {
					var name = school['fields']['name'];
					if (name.length > 20) {
						name = name.substr(0, 20) + '...';
					}
					return name;
				}),
				datasets: [{
					label: 'Receipts dataset',
					/*  fillColor: "rgba(102,204,153,0.9)",*/
					fillColor: "rgba(0,102,153,0.9)",
		            strokeColor: "rgba(255,255,255,0)",
		            highlightFill: "rgba(0,102,153,0.7)",
		            highlightStroke: "rgba(255,255,255,0)",
					data: field_data
				}]
			};
			var chart = new Chart(ctx).Bar(chartData);
		});
	}

	/**
	 * Init receipt chart
	 */
	function initReceiptChart() {
		initChart('receipts', '#receipts-chart');
	}

	function initSalariesChart() {
		initChart('salaries', '#salaries-chart', 'average_salary');
	}

	function initPaymentsPerStudentChart() {
		initChart('payments_per_student', '#payments_per_student-chart');
	}

	function initHighEducationIndexChart() {
		initChart('high_education_index', '#high_education_index-chart');
	}

	function initEgeIndexChart() {
		initChart('ege_index', '#ege_index-chart');
	}

	function initComputerIndexChart() {
		initChart('computer_index', '#computer_index-chart');
	}

	//------------------------------------------------------------------------------------------------------------------

	// Full rating

	var ratingViewModel = null;

	function initFullRating() {
		var ratingModalWindow = document.querySelector('#ratingModal');
		if (!ratingModalWindow) {
			return;
		}

		ratingViewModel = {
			title: ko.observable(''),
			schools: ko.observableArray([]),
			showSchoolInfo: function (school) {
				showSchoolPopup(school.id);
			}
		};
		ko.applyBindings(ratingViewModel, ratingModalWindow);
	}

	/**
	 * Show full rating by parameter
	 * @param parameter {string} School parameter
	 * @param [formatFunction] {function} format value function
	 */
	function showFullRating(parameter, formatFunction) {
		store.getAllSchoolsByParameter(parameter, function (data) {
			var ratingModalWindow = $('#ratingModal');
			if (!ratingModalWindow) {
				return;
			}

			var field = getFieldParameterUrl(parameter);
			if (!field) {
				return;
			}

			var schools = data.map(function (school, index) {
				return {
					id: school['pk'],
					name: school['fields']['name'],
					value: buildValue(school['fields'][field], formatFunction)
				};
			});

			ratingViewModel.title(getParameterName(parameter));
			ratingViewModel.schools(schools);

			ratingModalWindow.modal();
		});
	}

	/**
	 * Build value
	 * @param value {number}
	 * @param formatFunction {function}
	 * @returns {string}
	 */
	function buildValue (value, formatFunction) {
		if (!value) {
			return "Нет информации";
		}
		if (!formatFunction) {
			return value;
		}
		return formatFunction(value);
	}

	/**
	 * Get school field by parameter
	 * @param parameter {string}
	 * @returns {string}
	 */
	function getFieldParameterUrl (parameter) {
		switch (parameter) {
			case 'ege':
				return 'ege_index';
			case 'computers':
				return 'computer_index';
			case 'high-education-teachers':
				return 'high_education_index';
			case 'salaries':
				return 'average_salary';
			case 'payments-per-student':
				return 'payments_per_student';
			default:
				return parameter;
		}
	}

	/**
	 * Get parameter name
	 * @param parameter {string}
	 * @returns {string}
	 */
	function getParameterName (parameter) {
		switch (parameter) {
			case 'ege':
				return 'Доля учеников, сдавших ЕГЭ положительно';
			case 'computers':
				return 'Количество учеников на один компьютер';
			case 'high-education-teachers':
				return 'Доля учителей с высшим образованием';
			case 'salaries':
				return 'Средняя зарплата сотрудников школы';
			case 'payments-per-student':
				return 'Расходы на одного ученика';
			case 'receipts':
				return 'Бюджет';
			default:
				return '';
		}
	}

	var formatFunction = {
		currency: function (value) {
			return numeral(value).format('0,0 $');
		},
		percentage: function (value) {
			return value + '%';
		}
	};

	//------------------------------------------------------------------------------------------------------------------

	function generateParameterArrayForComparedSchool (parameterName) {
		return store.getComparedSchoolsList().map(function(elem) {
			return elem[parameterName];
		});
	}

	function updateViewModel () {
		compareViewModel.numberOfStudents(generateParameterArrayForComparedSchool('number_of_students_index'));
		compareViewModel.names(generateParameterArrayForComparedSchool('name'));
		compareViewModel.receipts(generateParameterArrayForComparedSchool('receipts'));
		compareViewModel.paymentsPerStudent(generateParameterArrayForComparedSchool('payments_per_student'));
		compareViewModel.averageSalaries(generateParameterArrayForComparedSchool('average_salary'));
		compareViewModel.highEducation(generateParameterArrayForComparedSchool('high_education_index'));
		compareViewModel.ege(generateParameterArrayForComparedSchool('ege_index'));
		compareViewModel.computersPerStudent(generateParameterArrayForComparedSchool('computer_index'));
		navigationViewModel.comparedSchoolsLength(store.getComparedSchoolsListLength());
	}

	// Compare schools
	var compareViewModel = {
		numberOfStudents: ko.observableArray([]),
		names: ko.observableArray([]),
		receipts: ko.observableArray([]),
		paymentsPerStudent: ko.observableArray([]),
		averageSalaries: ko.observableArray([]),
		highEducation: ko.observableArray([]),
		ege: ko.observableArray([]),
		computersPerStudent: ko.observableArray([]),
		removeFromComparedSchoolsList: function (index) {
			var schools = store.getComparedSchoolsList();
			if (!schools[index]) {
				return;
			}
			store.removeSchoolFromComparedList2(schools[index]);
			updateViewModel();
		}

	};

	function initCompareViewModel() {
		var compareElement = document.querySelector('#compare');
		if (!compareElement) {
			return;
		}
		updateViewModel();
		ko.applyBindings(compareViewModel, compareElement);
	}

	//------------------------------------------------------------------------------------------------------------------

	var currentSchool = null;

	var schoolViewModel = {
		name: ko.observable(''),
		address: ko.observable(''),
		receipts: ko.observable(0),
		averageSalary: ko.observable(0),
		computerIndex: ko.observable(0),
		egeIndex: ko.observable(0),
		highEducationIndex: ko.observable(0),
		numberOfStudents: ko.observable(0),
		payments: ko.observable(0),
		paymentsPerStudent: ko.observable(0),
		isSchoolInComparedList: ko.observable(false),
		addToComparedSchoolsList: function() {
			store.addSchoolToComparedList(currentSchool);
			this.isSchoolInComparedList(store.isSchoolInComparedList(currentSchool));
			navigationViewModel.comparedSchoolsLength(store.getComparedSchoolsListLength());
		},
		removeFromComparedSchoolsList: function() {
			store.removeSchoolFromComparedList2(currentSchool);
			this.isSchoolInComparedList(store.isSchoolInComparedList(currentSchool));
			navigationViewModel.comparedSchoolsLength(store.getComparedSchoolsListLength());
		}
	};

	function initSchoolInfoBindings() {
		var schoolModalWindow = document.querySelector('#schoolModal');
		if (!schoolModalWindow) {
			return;
		}
		ko.applyBindings(schoolViewModel, schoolModalWindow);
	}

	function showSchoolPopup (schoolId) {
		store.getSchoolInfo(schoolId, function (data) {
			var schoolModalWindow = $('#schoolModal');
			if (!schoolModalWindow) {
				return;
			}

			if (!data || !data[0] || !data[0]['fields']) {
				return;
			}

			// TODO refactor this
			var school = data[0]['fields'];
			school.id = data[0].pk;

			for (var field in school) {
				if (!school.hasOwnProperty(field)) {
					continue;
				}
				if (school[field] === null) {
					school[field] = '<abbr title="Нет информации">Н/И</abbr>';
					continue;
				}
				if (['receipts', 'average_salary', 'payments', 'payments_per_student'].indexOf(field) !== -1) {
					school[field] = formatFunction.currency(school[field]);
					continue;
				}
				if (['ege_index', 'high_education_index'].indexOf(field) !== -1) {
					school[field] = formatFunction.percentage(school[field]);
				}
			}

			currentSchool = school;

			schoolViewModel.name(school['name']);
			schoolViewModel.address(school['address']);
			schoolViewModel.receipts(school['receipts']);
			schoolViewModel.averageSalary(school['average_salary']);
			schoolViewModel.computerIndex(school['computer_index']);
			schoolViewModel.egeIndex(school['ege_index']);
			schoolViewModel.highEducationIndex(school['high_education_index']);
			schoolViewModel.numberOfStudents(school['number_of_students_index']);
			schoolViewModel.payments(school['payments']);
			schoolViewModel.paymentsPerStudent(school['payments_per_student']);
			schoolViewModel.isSchoolInComparedList(store.isSchoolInComparedList(currentSchool));

			schoolModalWindow.modal();
		});
	}

	//------------------------------------------------------------------------------------------------------------------

	/**
	 * Add placemark to map
	 * @param map {object} Yandex map object
	 * @param coordinates {number[]} Coordinates of placemark
	 * @param balloonContent {string} Balloon text of placemark
	 * @param school {Object} School object
	 */
	function addPlacemarkToMap (map, coordinates, balloonContent, school) {
		var placemark = new ymaps.Placemark(coordinates, {
			hintContent: balloonContent
		}, {
            iconLayout: 'default#image',
            // Своё изображение иконки метки.
            iconImageHref: '/static/media/icon.png',
            // Размеры метки.
            iconImageSize: [40, 40],
            // Смещение левого верхнего угла иконки относительно
            // её "ножки" (точки привязки).
            iconImageOffset: [-20, -20]
        });
		placemark.events.add('click', function() {
			showSchoolPopup(school.pk);
		});
		map.geoObjects.add(placemark);
	}

	function initMap() {

		if ($('#map').length === 0) {
			return;
		}

		var map = new ymaps.Map('map', {
			center: [53.355576, 83.734382],
			zoom: 13,
			controls: ['zoomControl'],
			behaviors: ['default']
		});
		map.behaviors.disable(['scrollZoom']);

		map.controls.add("geolocationControl", {
			float: "none", position: {bottom: 10, left: 10}
		});

		// get schools for map
		store.getSchoolsWithCoordinates(function(data) {
			var schools = data || [];
			schools.forEach(function(school) {
				addPlacemarkToMap(
					map,
					[school.fields.latitude, school.fields.longitude],
					school.fields.name,
					school
				)
			});
		});

	}

	if (typeof ymaps !== 'undefined') {
		ymaps.ready(function() {

			initMap();

		});
	}

	//------------------------------------------------------------------------------------------------------------------

	var navigationViewModel = {
		ratingLink: ko.computed(function() {
			if (document.querySelector('#scroll')) {
				return '#scroll';
			}
			return '/';
		}),
		comparedSchoolsLength: ko.observable(store.getComparedSchoolsListLength())
	};

	function initNavigationViewModel() {
		var navigation = document.querySelector('#navigation');
		if (!navigation) {
			return;
		}
		ko.applyBindings(navigationViewModel, navigation);
	}

	//------------------------------------------------------------------------------------------------------------------

	/**
	 * Init main bindings
	 */
	function bind() {

		var homeElement = document.querySelector('.home');
		if (homeElement) {
			ko.applyBindings({
				rating: {
					receipts: function() {
						showFullRating('receipts', formatFunction.currency);
						return false;
					},
					salaries: function() {
						showFullRating('salaries', formatFunction.currency);
						return false;
					},
					paymentsPerStudent: function() {
						showFullRating('payments-per-student', formatFunction.currency);
						return false;
					},
					highEducationIndex: function() {
						showFullRating('high-education-teachers', formatFunction.percentage);
						return false;
					},
					egeIndex: function() {
						showFullRating('ege', formatFunction.percentage);
						return false;
					},
					computerIndex: function() {
						showFullRating('computers');
						return false;
					}
				}
			}, homeElement);
		}

		initFullRating();
		initSchoolInfoBindings();
		initNavigationViewModel();
		initCompareViewModel();
	}

	$(function() {

		numeral.language('ru', {
			delimiters: {
				thousands: ' ',
				decimal: ','
			},
			abbreviations: {
				thousand: 'тыс.',
				million: 'млн',
				billion: 'млрд',
				trillion: 'трлн'
			},
			ordinal: function () {
				return '.';
			},
			currency: {
				symbol: '₽'
			}
		});

		$('.modal').on('hidden.bs.modal', function() {
			if ($('.modal.in').length) {
				$(document.body).addClass('modal-open')
			}
		});

		numeral.language('ru');

		$('a[href*=#]:not([href=#])').click(function () {
			if (location.pathname.replace(/^\//, '') == this.pathname.replace(/^\//, '') &&
				location.hostname == this.hostname) {
				var target = $(this.hash);
				target = target.length ? target : $('[name=' + this.hash.slice(1) + ']');
				if (target.length) {
					$('html,body').animate({
                        scrollTop: target.offset().top
                    }, 1000);
					return false;
				}
			}
		});

		bind();
		new WOW().init();
		initCharts();

	});


})($, ko, window.store);